@@ -45,6 +45,7 @@ class DBTester(object):
         if close_connection:
             connection.close()
 
+
 # silly mock class
 class Mock(object):
     pass
@@ -247,6 +248,12 @@ class DBConnectionPool(DBTester):
         self.pool.clear()
         self.assertEqual(len(self.pool.free_items), 0)
 
+    def test_clear_warmup(self):
+        """Clear implicitly created connections (min_size > 0)"""
+        self.pool = self.create_pool(min_size=1)
+        self.pool.clear()
+        self.assertEqual(len(self.pool.free_items), 0)
+
     def test_unwrap_connection(self):
         self.assert_(isinstance(self.connection,
                                 db_pool.GenericConnectionWrapper))
@@ -431,6 +438,15 @@ class DBConnectionPool(DBTester):
         self.assertEquals(self.pool.free(), 1)
 
 
+class DummyConnection(object):
+    pass
+
+
+class DummyDBModule(object):
+    def connect(self, *args, **kwargs):
+        return DummyConnection()
+
+
 class RaisingDBModule(object):
     def connect(self, *args, **kw):
         raise RuntimeError()
@@ -438,12 +454,12 @@ class RaisingDBModule(object):
 
 class TpoolConnectionPool(DBConnectionPool):
     __test__ = False  # so that nose doesn't try to execute this directly
-    def create_pool(self, max_size=1, max_idle=10, max_age=10,
+    def create_pool(self, min_size=0, max_size=1, max_idle=10, max_age=10,
                     connect_timeout=0.5, module=None):
         if module is None:
             module = self._dbmodule
         return db_pool.TpooledConnectionPool(module,
-            min_size=0, max_size=max_size,
+            min_size=min_size, max_size=max_size,
             max_idle=max_idle, max_age=max_age,
             connect_timeout = connect_timeout,
             **self._auth)
@@ -462,18 +478,30 @@ class TpoolConnectionPool(DBConnectionPool):
 
 class RawConnectionPool(DBConnectionPool):
     __test__ = False  # so that nose doesn't try to execute this directly
-    def create_pool(self, max_size=1, max_idle=10, max_age=10,
+    def create_pool(self, min_size=0, max_size=1, max_idle=10, max_age=10,
                     connect_timeout=0.5, module=None):
         if module is None:
             module = self._dbmodule
         return db_pool.RawConnectionPool(module,
-            min_size=0, max_size=max_size,
+            min_size=min_size, max_size=max_size,
             max_idle=max_idle, max_age=max_age,
             connect_timeout=connect_timeout,
             **self._auth)
 
+
+class TestRawConnectionPool(TestCase):
+    def test_issue_125(self):
+        # pool = self.create_pool(min_size=3, max_size=5)
+        pool = db_pool.RawConnectionPool(DummyDBModule(),
+            dsn="dbname=test user=jessica port=5433",
+            min_size=3, max_size=5)
+        conn = pool.get()
+        pool.put(conn)
+
+
 get_auth = get_database_auth
 
+
 def mysql_requirement(_f):
     verbose = os.environ.get('eventlet_test_mysql_verbose')
     try:
@@ -492,6 +520,7 @@ def mysql_requirement(_f):
             print >> sys.stderr, ">> Skipping mysql tests, MySQLdb not importable"
         return False
 
+
 class MysqlConnectionPool(object):
     dummy_table_sql = """CREATE TEMPORARY TABLE test_table
         (
@@ -539,9 +568,11 @@ class MysqlConnectionPool(object):
 class Test01MysqlTpool(MysqlConnectionPool, TpoolConnectionPool, TestCase):
     __test__ = True
 
+
 class Test02MysqlRaw(MysqlConnectionPool, RawConnectionPool, TestCase):
     __test__ = True
 
+
 def postgres_requirement(_f):
     try:
         import psycopg2
@@ -606,11 +637,14 @@ class Psycopg2ConnectionPool(object):
         db.close()
         del db
 
+
 class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestCase):
     __test__ = True
 
+
 class Test02Psycopg2Raw(Psycopg2ConnectionPool, RawConnectionPool, TestCase):
     __test__ = True
 
+
 if __name__ == '__main__':
     main()
