@@ -2,7 +2,7 @@ import socket as _orig_sock
 from tests import LimitedTestCase, skip_with_pyevent, main, skipped, s2b, skip_if, skip_on_windows
 from eventlet import event, greenio, debug
 from eventlet.hubs import get_hub
-from eventlet.green import socket, time
+from eventlet.green import select, socket, time
 from eventlet.support import get_errno
 
 import array
@@ -40,6 +40,13 @@ def using_epoll_hub(_f):
             return False
 
 
+def using_kqueue_hub(_f):
+        try:
+            return 'kqueue' in type(get_hub()).__module__
+        except Exception:
+            return False
+
+
 class TestGreenSocket(LimitedTestCase):
     def assertWriteToClosedFileRaises(self, fd):
         if sys.version_info[0] < 3:
@@ -182,6 +189,7 @@ class TestGreenSocket(LimitedTestCase):
         gt.wait()
 
     def test_send_timeout(self):
+        self.reset_timeout(2)
         listener = bufsized(eventlet.listen(('', 0)))
 
         evt = event.Event()
@@ -489,6 +497,7 @@ class TestGreenSocket(LimitedTestCase):
 
     @skip_with_pyevent
     @skip_if(using_epoll_hub)
+    @skip_if(using_kqueue_hub)
     def test_closure(self):
         def spam_to_me(address):
             sock = eventlet.connect(address)
@@ -581,6 +590,18 @@ class TestGreenSocket(LimitedTestCase):
         flags = fcntl.fcntl(sock2.fd.fileno(), fcntl.F_GETFL)
         assert flags & os.O_NONBLOCK == 0
 
+    def test_sockopt_interface(self):
+        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        assert sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) == 0
+        assert sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1) == '\000'
+        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+
+    def test_socketpair_select(self):
+        # https://github.com/eventlet/eventlet/pull/25
+        s1, s2 = socket.socketpair()
+        assert select.select([], [s1], [], 0) == ([], [s1], [])
+        assert select.select([], [s1], [], 0) == ([], [s1], [])
+
 
 class TestGreenPipe(LimitedTestCase):
     @skip_on_windows
