@@ -285,7 +285,7 @@ class Subprocess(ProcessBase):
 eventlet.monkey_patch()
 from eventlet.green import subprocess
 
-subprocess.Popen(['/bin/true'], stdin=subprocess.PIPE)
+subprocess.Popen(['true'], stdin=subprocess.PIPE)
 print "done"
 """
         self.write_to_tempfile("newmod", new_mod)
@@ -293,5 +293,196 @@ print "done"
         self.assertEqual(output, "done\n", output)
 
 
+class Threading(ProcessBase):
+    def test_orig_thread(self):
+        new_mod = """import eventlet
+eventlet.monkey_patch()
+from eventlet import patcher
+import threading
+_threading = patcher.original('threading')
+def test():
+    print repr(threading.currentThread())
+t = _threading.Thread(target=test)
+t.start()
+t.join()
+print len(threading._active)
+print len(_threading._active)
+"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 4, "\n".join(lines))
+        self.assert_(lines[0].startswith('<Thread'), lines[0])
+        self.assertEqual(lines[1], "1", lines[1])
+        self.assertEqual(lines[2], "1", lines[2])
+
+    def test_threading(self):
+        new_mod = """import eventlet
+eventlet.monkey_patch()
+import threading
+def test():
+    print repr(threading.currentThread())
+t = threading.Thread(target=test)
+t.start()
+t.join()
+print len(threading._active)
+"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assert_(lines[0].startswith('<_MainThread'), lines[0])
+        self.assertEqual(lines[1], "1", lines[1])
+
+    def test_tpool(self):
+        new_mod = """import eventlet
+eventlet.monkey_patch()
+from eventlet import tpool
+import threading
+def test():
+    print repr(threading.currentThread())
+tpool.execute(test)
+print len(threading._active)
+"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assert_(lines[0].startswith('<Thread'), lines[0])
+        self.assertEqual(lines[1], "1", lines[1])
+
+    def test_greenlet(self):
+        new_mod = """import eventlet
+eventlet.monkey_patch()
+from eventlet import event
+import threading
+evt = event.Event()
+def test():
+    print repr(threading.currentThread())
+    evt.send()
+eventlet.spawn_n(test)
+evt.wait()
+print len(threading._active)
+"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assert_(lines[0].startswith('<_MainThread'), lines[0])
+        self.assertEqual(lines[1], "1", lines[1])
+
+    def test_greenthread(self):
+        new_mod = """import eventlet
+eventlet.monkey_patch()
+import threading
+def test():
+    print repr(threading.currentThread())
+t = eventlet.spawn(test)
+t.wait()
+print len(threading._active)
+"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assert_(lines[0].startswith('<_GreenThread'), lines[0])
+        self.assertEqual(lines[1], "1", lines[1])
+
+    def test_keyerror(self):
+        new_mod = """import eventlet
+eventlet.monkey_patch()
+"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 1, "\n".join(lines))
+
+
+class Os(ProcessBase):
+    def test_waitpid(self):
+        new_mod = """import subprocess
+import eventlet
+eventlet.monkey_patch(all=False, os=True)
+process = subprocess.Popen("sleep 0.1 && false", shell=True)
+print process.wait()"""
+        self.write_to_tempfile("newmod", new_mod)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 2, "\n".join(lines))
+        self.assertEqual('1',  lines[0], repr(output))
+
+
+class GreenThreadWrapper(ProcessBase):
+    prologue = """import eventlet
+eventlet.monkey_patch()
+import threading
+def test():
+    t = threading.currentThread()
+"""
+    epilogue = """
+t = eventlet.spawn(test)
+t.wait()
+"""
+
+    def test_join(self):
+        self.write_to_tempfile("newmod", self.prologue + """
+    def test2():
+        global t2
+        t2 = threading.currentThread()
+    eventlet.spawn(test2)
+""" + self.epilogue + """
+print repr(t2)
+t2.join()
+""")
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 2, "\n".join(lines))
+        self.assert_(lines[0].startswith('<_GreenThread'), lines[0])
+
+    def test_name(self):
+        self.write_to_tempfile("newmod", self.prologue + """
+    print t.name
+    print t.getName()
+    print t.get_name()
+    t.name = 'foo'
+    print t.name
+    print t.getName()
+    print t.get_name()
+    t.setName('bar')
+    print t.name
+    print t.getName()
+    print t.get_name()
+""" + self.epilogue)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 10, "\n".join(lines))
+        for i in xrange(0, 3):
+            self.assertEqual(lines[i], "GreenThread-1", lines[i])
+        for i in xrange(3, 6):
+            self.assertEqual(lines[i], "foo", lines[i])
+        for i in xrange(6, 9):
+            self.assertEqual(lines[i], "bar", lines[i])
+
+    def test_ident(self):
+        self.write_to_tempfile("newmod", self.prologue + """
+    print id(t._g)
+    print t.ident
+""" + self.epilogue)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assertEqual(lines[0], lines[1])
+
+    def test_is_alive(self):
+        self.write_to_tempfile("newmod", self.prologue + """
+    print t.is_alive()
+    print t.isAlive()
+""" + self.epilogue)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assertEqual(lines[0], "True", lines[0])
+        self.assertEqual(lines[1], "True", lines[1])
+
+    def test_is_daemon(self):
+        self.write_to_tempfile("newmod", self.prologue + """
+    print t.is_daemon()
+    print t.isDaemon()
+""" + self.epilogue)
+        output, lines = self.launch_subprocess('newmod')
+        self.assertEqual(len(lines), 3, "\n".join(lines))
+        self.assertEqual(lines[0], "True", lines[0])
+        self.assertEqual(lines[1], "True", lines[1])
+
+
 if __name__ == '__main__':
     main()
